#include "XY.h"

//double SetpointX, InputX, OutputX, SetpointY, InputY, OutputY;

XY::XY(int nSampleTime)
{
  pwm_x = 6;  //PWM control for motor outputs 1 and 2 
  pwm_y = 9;  //PWM control for motor outputs 3 and 4 
  dir_x = 7;  //direction control for motor outputs 1 and 2 
  dir_y = 8;  //direction control for motor outputs 3 and 4 
  pos_x = 1; //position feedback for motor 1/2
  pos_y = 0; //position feedback for motor 3/4
  
  xy_state = XY_IDLE;

  pinMode(pwm_x, OUTPUT);  //Set control pins to be outputs
  pinMode(pwm_y, OUTPUT);
  pinMode(dir_x, OUTPUT);
  pinMode(dir_y, OUTPUT);

  //initialize the variables to current position so it doesn't move at start
  InputX = analogRead(pos_x);
  SetpointX = InputX;
  InputY = analogRead(pos_y);
  SetpointY = InputY;

  PIDX.SetLinks(&InputX, &OutputX, &SetpointX);
  PIDY.SetLinks(&InputY, &OutputY, &SetpointY);
  // working (within +/- 5) tunings: (5,0,0)
  // y-axis seems fine with (0.1,0.1,0)
  PIDX.SetTunings(0.1,0.1,0);
  PIDY.SetTunings(0.1,0.1,0);
  PIDX.SetSampleTime(nSampleTime);
  PIDY.SetSampleTime(nSampleTime);
  PIDX.SetControllerDirection(DIRECT);
  PIDY.SetControllerDirection(DIRECT);
  PIDX.SetMode(AUTOMATIC);
  PIDY.SetMode(AUTOMATIC);
}

int XY::check()
{
	int state = 0;
	
    // enter deadzone if setpoint is reached:
    if (abs(InputX-SetpointX)>DEADZONE_X) {
      InputX = analogRead(pos_x);
      PIDX.Compute();
      if (InputX > SetpointX) {
        digitalWrite(dir_x, LOW);
        PIDX.SetControllerDirection(REVERSE);
      } else {
        digitalWrite(dir_x, HIGH);
        PIDX.SetControllerDirection(DIRECT);
      }
      analogWrite(pwm_x,OutputX);
    }
    else
    {
      analogWrite(pwm_x,0);
	  state++;
    }
    
    // enter deadzone if setpoint is reached:
    if (abs(InputY-SetpointY)>DEADZONE_Y) {
      InputY = analogRead(pos_y);
      PIDY.Compute();
      if (InputY > SetpointY) {
        digitalWrite(dir_y, LOW);
        PIDY.SetControllerDirection(REVERSE);
      } else {
        digitalWrite(dir_y, HIGH);
        PIDY.SetControllerDirection(DIRECT);
      }
      analogWrite(pwm_y,OutputY);
    }
    else
    {
      analogWrite(pwm_y,0);
	  state++;
    }
	
	if (state == 2) {xy_state = XY_IDLE;}
	else {xy_state = XY_MOVING;}
	return xy_state;
}