/*
XY.h - Arduino library for supporting an XY arm based on Firgelli linear actuators and Velleman VMA03 motor shield
Copyright (c) 2015 Ren Becker.  All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ARDUINO_XY
#define ARDUINO_XY

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif
#include <PID.h>

#define XY_LIBRARY_VERSION 1
// xy_state values:
#define XY_IDLE 0
#define XY_MOVING 1
// x and y bounds:
#define MAX_X 1000
#define MAX_Y 1000
#define MIN_X 25
#define MIN_Y 25
#define DEADZONE_X 1
#define DEADZONE_Y 1
#define TRAVERSE_X 13 // cm
#define TRAVERSE_Y 9.5 // cm


class XY
{
  public:
    XY(int nSampleTime);
    int check();
	double getSetpointX() const { return SetpointX; }
	double getSetpointY() const { return SetpointY; }
	double getInputX() const { return InputX; }
	double getInputY() const { return InputY; }
	double getOutputX() const { return OutputX; }
	double getOutputY() const { return OutputY; }
	void setSetpointX(const double _SetpointX) { SetpointX = _SetpointX; }
	void setSetpointY(const double _SetpointY) { SetpointY = _SetpointY; }
	void setInputX(const double _InputX) { InputX = _InputX; }
	void setInputY(const double _InputY) { InputY = _InputY; }
	void setOutputX(const double _OutputX) { OutputX = _OutputX; }
	void setOutputY(const double _OutputY) { OutputY = _OutputY; }
	bool setXcm(const double _SetXcm) { SetpointX = MIN_X+int(_SetXcm*double(MAX_X-MIN_X)/TRAVERSE_X); }
	bool setYcm(const double _SetYcm) { SetpointY = MIN_Y+int(_SetYcm*double(MAX_Y-MIN_Y)/TRAVERSE_Y); }
	double getXcm() { return TRAVERSE_X*double(InputX-MIN_X)/double(MAX_X-MIN_X); }
	double getYcm() { return TRAVERSE_Y*double(InputY-MIN_Y)/double(MAX_Y-MIN_Y); }
	
  private:
    double SetpointX, InputX, OutputX, SetpointY, InputY, OutputY, DeadzoneX, DeadzoneY;
    PID PIDX, PIDY;
	int xy_state;
    int pwm_x;  //PWM control for motor outputs 1 and 2 
    int pwm_y;  //PWM control for motor outputs 3 and 4 
    int dir_x;  //direction control for motor outputs 1 and 2 
    int dir_y;  //direction control for motor outputs 3 and 4 
    int pos_x; //position feedback for motor 1/2
    int pos_y; //position feedback for motor 3/4
};




#endif
