#include <XY.h>
#include <Wire.h>
#include <LCD03.h>
#include <Streaming.h>
#include <Metro.h>
#include <PID.h>

/* sequence setup */
#define NUM_CYCLES 10             // number of cycles in the sequence
#define NUM_STEP 3                // number of steps in each cycle
float x_dist[NUM_STEP] = {0,2,2}; // in cm (relative to current position; negative also possible)
float x_wait[NUM_STEP] = {5,5,5}; // in sec
#define X_START 0                 // X starting position (in cm)
#define Y_UP 0                    // Y up position (in cm)
#define Y_DOWN 8                  // U down position (in cm)
/* end sequence setup */

#define PRG_INIT 0
#define PRG_RUNNING 1
#define PRG_FINISHED 2
#define PRG_TERMINATED 3
#define XY_UP 0
#define XY_DOWN 1
#define XY_MOVING_UP 2
#define XY_MOVING_DOWN 3
int DipCoaterState = XY_IDLE;
int ProgramState = PRG_INIT;
int XYState = XY_UP;
boolean dipping = false;

// LCD and LCD scheduler instantiation
LCD03 lcd;
Metro metroLCD = Metro(200);
// dip coater instantiation
int SampleTime = 100;
Metro metroDipCoater = Metro(SampleTime);
XY dipcoater(SampleTime);
// scheduler instantiation
Metro metroScheduler = Metro(20000);
Metro metroCounter = Metro(1000);
int counter = -1;

int x_step = 0;
int cycle = 0;
int timer = 0;

void setup()
{
  //Serial.begin(9600);
  //Serial.println("asd");
  
  lcd.begin(20, 4);
  lcd.backlight();
  lcd.clear();
  
  //analogReference(EXTERNAL);
  
  randomSeed(analogRead(3));
}

void loop()
{
    
  // get input to control dip coater
  if (metroScheduler.check() == 1)
  {
    // make random moves around the whole range:
    //dipcoater.setXcm(random(0, TRAVERSE_X));
    //dipcoater.setYcm(random(0, TRAVERSE_Y));
    //dipcoater.setYcm(TRAVERSE_Y);

  }
  
  // run control loop
  if (metroDipCoater.check() == 1)
  {
    DipCoaterState = dipcoater.check();
    
    switch (DipCoaterState) 
    {
      case XY_IDLE:
        switch(ProgramState)
        {
          case PRG_INIT:
            dipcoater.setXcm(X_START);
            dipcoater.setYcm(Y_UP);
            ProgramState = PRG_RUNNING;
            break;
          case PRG_RUNNING:
            if (cycle < NUM_CYCLES)
            {
              if (x_step < NUM_STEP)
              {
                switch (XYState)
                {
                  case XY_UP:
                    dipcoater.setXcm(dipcoater.getXcm()+x_dist[x_step]);
                    XYState = XY_MOVING_DOWN;
                    break;
                  case XY_MOVING_DOWN:
                    dipcoater.setYcm(Y_DOWN);
                    XYState = XY_DOWN;
                    break;
                  case XY_DOWN:
                    if (counter == -1)
                    {
                      counter = 0;
                      dipping = true;
                    }
                    else if (counter > x_wait[x_step])
                    {
                      counter = -1;
                      dipping = false;
                      XYState = XY_MOVING_UP;
                    }
                    break;
                  case XY_MOVING_UP:
                    x_step++;
                    dipcoater.setYcm(Y_UP);
                    XYState = XY_UP;
                    break;
                }
              }
              else
              {
                x_step = 0;
                dipcoater.setXcm(X_START);
                cycle++;
              }
            }
            else
            {
              ProgramState = PRG_FINISHED;
            }
            break;
          case PRG_FINISHED:
            break;
          case PRG_TERMINATED:
            break;
        }
        break;
      case XY_MOVING:
        break;
    }
  }
  
  // print output to LCD
  if (metroLCD.check() == 1)
  {
    lcd.clear();
    lcd.setCursor(0,0);
    if (dipping)
      lcd << "Dipping " << _FLOAT(counter,0) << "/" << _FLOAT(x_wait[x_step],0) << " s";
    else if (DipCoaterState == XY_MOVING)
      lcd << "Moving to position..";
    else if (DipCoaterState == XY_IDLE)
      lcd << "At position!";
    
    lcd.setCursor(0,1);
    lcd << "X:" << _FLOAT(dipcoater.getInputX(),0) << "/" << _FLOAT(dipcoater.getSetpointX(),0) <<
      " Y:" << _FLOAT(dipcoater.getInputY(),0) << "/" << _FLOAT(dipcoater.getSetpointY(),0);
    lcd.setCursor(0,2);
    lcd << dipcoater.getOutputX() << " " << dipcoater.getOutputY();
    lcd.setCursor(0,3);
    lcd << "DC" << DipCoaterState << " PRG" << ProgramState << " XY" << XYState << " S" << x_step << " C" << cycle;
  }
  
  if (metroCounter.check() == 1)
  {
    if (counter >= 0)
      counter++;
  }
}


